"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.bootstrap = void 0;
const container_1 = require("./container");
const enum_1 = require("./constants/enum");
const json_1 = require("./helpers/json");
const joi_1 = __importDefault(require("joi"));
const constants_1 = require("./constants/constants");
/**
 * Bootstraps the whole application.
 * @param params Register all the controllers here and optionally provide a base route for all endpoints.
 */
function bootstrap(params) {
    params.controllers.forEach(x => {
        const controllerName = x.name.replace('Controller', '').toLowerCase();
        const controllerInstance = new x();
        container_1.ControllersContainer.set(controllerName, controllerInstance);
    });
    const trimRegex = /(^\/)|(\/$)/g;
    params.base = params.base ? `/${params.base.replace(trimRegex, '')}` : '';
    return (req, res, next) => {
        let isRouteResolved = false;
        let validationErrorOccurred = false;
        const url = req.url.replace(params.base, '');
        const method = req.method;
        const controllerName = url.split('/')[1].split('?')[0];
        const queryParams = req.query;
        const requestBody = req.body;
        const controller = container_1.ControllersContainer.get(controllerName);
        if (controller) {
            const routes = controller.routes;
            const matchedRoute = routes.find(x => new RegExp(x.routeRegex).test(url)
                && x.httpMethod.toString() === method);
            if (matchedRoute && req.url.startsWith(params.base)) {
                const routeParams = getRouteParams(matchedRoute.route, matchedRoute.routeParamsIndices, url);
                let methodExecutionExpression = `${Object.keys({ controller })[0]}.${matchedRoute.method}(`;
                let modelParams = new Array();
                const paramsArrayVarName = Object.keys({ modelParams })[0];
                matchedRoute.params.forEach((x, index) => {
                    let validationError;
                    switch (x.paramType) {
                        case enum_1.ParamType.Body:
                            validationError = performInputValidation(x.model, requestBody);
                            if (validationError) {
                                res.status(enum_1.HTTPResponse.UnprocessableEntity).send(validationError);
                                validationErrorOccurred = true;
                            }
                            else {
                                modelParams.push(json_1.deserialize(requestBody, x.model));
                                methodExecutionExpression += `${paramsArrayVarName}[${modelParams.length - 1}]`;
                            }
                            break;
                        case enum_1.ParamType.Query:
                            validationError = performInputValidation(x.model, queryParams);
                            if (validationError) {
                                res.status(enum_1.HTTPResponse.UnprocessableEntity).send(validationError);
                                validationErrorOccurred = true;
                            }
                            else {
                                modelParams.push(json_1.deserialize(queryParams, x.model));
                                methodExecutionExpression += `${paramsArrayVarName}[${modelParams.length - 1}]`;
                            }
                            break;
                        case enum_1.ParamType.Route:
                            methodExecutionExpression += isNaN(routeParams[x.name]) ?
                                `'${decodeURIComponent(routeParams[x.name])}'` : routeParams[x.name];
                            break;
                    }
                    if (index < matchedRoute.params.length - 1) {
                        methodExecutionExpression += ',';
                    }
                });
                methodExecutionExpression += ')';
                if (!validationErrorOccurred) {
                    const result = eval(methodExecutionExpression);
                    // Check if result has the custom 'Response' type.
                    if (result[constants_1.responseClassIdentifier]) {
                        res.status(result.statusCode).send(result.data);
                    }
                    else {
                        res.status(200).send(result);
                    }
                }
                isRouteResolved = true;
            }
        }
        if (!isRouteResolved) {
            res.status(404).send('Not Found');
        }
    };
}
exports.bootstrap = bootstrap;
function getRouteParams(toMatch, routeParamsIndices, toParse) {
    const toMatchSplit = toMatch.split('/');
    const toParseSplit = toParse.split('/');
    const routeParams = new Object();
    routeParamsIndices.forEach(index => {
        const queryParamsRegex = /[?].*/;
        routeParams[toMatchSplit[index].replace(':', '')] =
            toParseSplit[index].replace(queryParamsRegex, '');
    });
    return routeParams;
}
function performInputValidation(model, input) {
    let validationErrorDetails;
    const obj = new model();
    if (obj[constants_1.validationSchemaPropertyName]) {
        const validationError = joi_1.default.object(obj[constants_1.validationSchemaPropertyName]).validate(input).error;
        if (validationError) {
            validationErrorDetails = validationError.details;
        }
    }
    return validationErrorDetails;
}
