"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FromRoute = exports.FromBody = exports.FromQuery = exports.Delete = exports.Patch = exports.Put = exports.Post = exports.Get = void 0;
const enum_1 = require("../constants/enum");
const paramInfo_1 = require("../entities/paramInfo");
function checkAndCreateRoutesProperty(target) {
    if (!target.hasOwnProperty('routes')) {
        Object.defineProperty(target, 'routes', {
            value: new Array()
        });
    }
}
function checkAndCreateRoute(target, routes, key) {
    let route = routes.find(x => x.method === key);
    if (!route) {
        route = {
            method: key,
            httpMethod: enum_1.HTTPMethod.Get,
            params: new Array(),
            route: '',
            routeRegex: '',
            routeParamsIndices: []
        };
        target.routes.push(route);
    }
    return route;
}
/**
 * Specifies that a controller method is to be exposed as an endpoint.
 * @param route Optional route for the method to be exposed as endpoint.
 * If not provided, assumes the default for this resource.
 * @param httpMethod Optional http method for the route.
 * If not provided, assumes the default which is 'GET'.
 */
function routeHandler(route, httpMethod) {
    return (target, propertyKey) => {
        // if (!target.hasOwnProperty('routes')) {
        //     Object.defineProperty(target, 'routes', {
        //         value: new Array<RouteInfo>()
        //     });
        // }
        checkAndCreateRoutesProperty(target);
        const controllerName = target.constructor.name.replace('Controller', '').toLowerCase();
        route = route ? route : '';
        route = `/${controllerName}${route}`;
        httpMethod = httpMethod ? httpMethod : enum_1.HTTPMethod.Get;
        const routeParamsIndices = new Array();
        const splitRoute = route.split('/');
        for (let i = 0; i < splitRoute.length; i++) {
            if (splitRoute[i].startsWith(':')) {
                routeParamsIndices.push(i);
            }
        }
        const queriesOrNone = '(?:[?]{1}.*|)';
        const alphaNumericAny = '[0-9A-Za-z]+';
        // const alphaNumericAny = '(?!.*(\\/|#|\\?)).+';
        const routeParamRegex = /:[0-9A-Za-z]+/g;
        const routes = target.routes;
        const matchedRoute = routes.find(x => x.method === propertyKey);
        if (matchedRoute) {
            matchedRoute.route = route;
            matchedRoute.httpMethod = httpMethod;
            matchedRoute.routeRegex = '^' + route.replace(/\//g, '\\/').replace(routeParamRegex, alphaNumericAny) + queriesOrNone + '$';
            matchedRoute.routeParamsIndices = routeParamsIndices;
        }
        else {
            target.routes.push({
                route: route,
                routeRegex: '^' + route.replace(/\//g, '\\/').replace(routeParamRegex, alphaNumericAny) + queriesOrNone + '$',
                method: propertyKey,
                httpMethod: httpMethod,
                routeParamsIndices: routeParamsIndices,
                params: new Array()
            });
        }
    };
}
/**
 * Specifies that a controller method is to be exposed as an HTTP GET endpoint.
 * @param route Optional route for the method to be exposed as endpoint.
 * If not provided, assumes the default for this resource.
 */
function Get(route) {
    return routeHandler(route, enum_1.HTTPMethod.Get);
}
exports.Get = Get;
/**
 * Specifies that a controller method is to be exposed as an HTTP POST endpoint.
 * @param route Optional route for the method to be exposed as endpoint.
 * If not provided, assumes the default for this resource.
 */
function Post(route) {
    return routeHandler(route, enum_1.HTTPMethod.Post);
}
exports.Post = Post;
/**
 * Specifies that a controller method is to be exposed as an HTTP PUT endpoint.
 * @param route Optional route for the method to be exposed as endpoint.
 * If not provided, assumes the default for this resource.
 */
function Put(route) {
    return routeHandler(route, enum_1.HTTPMethod.Put);
}
exports.Put = Put;
/**
 * Specifies that a controller method is to be exposed as an HTTP PATCH endpoint.
 * @param route Optional route for the method to be exposed as endpoint.
 * If not provided, assumes the default for this resource.
 */
function Patch(route) {
    return routeHandler(route, enum_1.HTTPMethod.Patch);
}
exports.Patch = Patch;
/**
 * Specifies that a controller method is to be exposed as an HTTP DELETE endpoint.
 * @param route Optional route for the method to be exposed as endpoint.
 * If not provided, assumes the default for this resource.
 */
function Delete(route) {
    return routeHandler(route, enum_1.HTTPMethod.Delete);
}
exports.Delete = Delete;
/**
 * When set on a route parameter, specifies that the parameter comes from the query of the request.
 */
function FromQuery(model) {
    return (target, functionKey, parameterIndex) => {
        checkAndCreateRoutesProperty(target);
        const routes = target.routes;
        let route = checkAndCreateRoute(target, routes, functionKey);
        if (route) {
            if (route.params.find(x => x.paramType === enum_1.ParamType.Query)) {
                throw new Error('There cannot be multiple query param models in a single request.');
            }
            route.params =
                [new paramInfo_1.ParamInfo(parameterIndex, '', enum_1.ParamType.Query, model), ...route.params];
        }
    };
}
exports.FromQuery = FromQuery;
/**
 * When set on a route parameter, specifies that the parameter comes from the body of the request.
 */
function FromBody(model) {
    return (target, functionKey, parameterIndex) => {
        checkAndCreateRoutesProperty(target);
        const routes = target.routes;
        let route = checkAndCreateRoute(target, routes, functionKey);
        if (route) {
            if (route.params.find(x => x.paramType === enum_1.ParamType.Body)) {
                throw new Error('There cannot be multiple bodies in a single request.');
            }
            route.params =
                [new paramInfo_1.ParamInfo(parameterIndex, '', enum_1.ParamType.Body, model), ...route.params];
        }
    };
}
exports.FromBody = FromBody;
/**
 * When set on a route parameter, specifies that the parameter comes from the route of the request.
 * @param parameterName Parameter name to be used. Note that the parameter name specified here should match with the one in the route.
 */
function FromRoute(parameterName) {
    return function (target, functionKey, parameterIndex) {
        checkAndCreateRoutesProperty(target);
        const routes = target.routes;
        let route = checkAndCreateRoute(target, routes, functionKey);
        if (route) {
            route.params =
                [new paramInfo_1.ParamInfo(parameterIndex, parameterName, enum_1.ParamType.Route), ...route.params];
        }
    };
}
exports.FromRoute = FromRoute;
